import os
from urllib.parse import urlparse

from loguru import logger
from pyrogram import Client
from ruamel.yaml import YAML

from src.exceptions.download_exceptions import FileAlreadyExistException
from src.parsers.Telegram.telegram_media_downloader.media_downloader import _check_config, app, download_all_chat, \
    worker
from src.parsers.base_parser import BaseParser

import asyncio
import threading
from typing import Awaitable, TypeVar
T = TypeVar("T")


def _start_background_loop(loop):
    asyncio.set_event_loop(loop)
    loop.run_forever()


_LOOP = asyncio.new_event_loop()
_LOOP_THREAD = threading.Thread(
    target=_start_background_loop, args=(_LOOP,), daemon=True
)
_LOOP_THREAD.start()


def asyncio_run(coro: Awaitable[T], timeout=30) -> T:
    """
    Runs the coroutine in an event loop running on a background thread,
    and blocks the current thread until it returns a result.
    This plays well with gevent, since it can yield on the Future result call.

    :param coro: A coroutine, typically an async method
    :param timeout: How many seconds we should wait for a result before raising an error
    """
    return asyncio.run_coroutine_threadsafe(coro, _LOOP).result(timeout=timeout)


def asyncio_gather(*futures, return_exceptions=False) -> list:
    """
    A version of asyncio.gather that runs on the internal event loop
    """
    async def gather():
        return await asyncio.gather(*futures, return_exceptions=return_exceptions)

    return asyncio.run_coroutine_threadsafe(gather(), loop=_LOOP).result()


class TelegramParser(BaseParser):
    def video_download(self, client: Client = None):
        url_parse_result = urlparse(self.params["link"])
        channel, message_id = url_parse_result.path[1:].split('/') if "/c/" not in url_parse_result.path else \
            url_parse_result.path[3:].split('/')
        if os.path.exists(os.path.join(os.getcwd() + f"/downloads/Telegram/{message_id}.mp4")):
            raise FileAlreadyExistException(message=f"Telegram/{message_id}.mp4")
        with open(os.path.join(os.path.abspath(""), "src/parsers/Telegram/telegram_media_downloader/config.yaml"),
                  mode="r+", encoding="utf-8") as f:
            config = YAML().load(f.read())
            config["chat"][0]['download_filter'] = f"id == {message_id}"
            config["chat"][0]['chat_id'] = channel if "/c/" not in url_parse_result.path else int(f"-100{channel}")
            config["chat"][0]['last_read_message_id'] = int(message_id)

        with open(os.path.join(os.path.abspath(""), "src/parsers/Telegram/telegram_media_downloader/config.yaml"),
                  mode="w+", encoding="utf-8") as f:
            YAML().dump(config, f)
        if _check_config():
            a = asyncio_run(download_all_chat(client))
            b = asyncio_run(worker(client))
            client.stop()
            app.is_running = False
            logger.info("Stopped!")
        return f"Telegram/{message_id}.mp4"
